"""Hacth custom hook module."""
# pylint: disable=import-error,too-few-public-methods,wrong-import-position

import os
import sys

from hatchling.builders.hooks.plugin.interface import BuildHookInterface

sys.path.append(os.path.dirname(__file__))


class CustomWheelHook(BuildHookInterface):
    """Custom wheel hook to set correct build data."""

    def initialize(self, _, build_data):
        """Will be called before creating the source archive."""

        build_data['artifacts'] = [
            os.path.join('qrkey', 'ui', 'build'),
        ]



# SPDX-FileCopyrightText: 2024-present Alexandre Abadie <alexandre.abadie@inria.fr>
#
# SPDX-License-Identifier: BSD-3-Clause

import asyncio
import io
from unittest.mock import MagicMock

import pytest  # type: ignore
import segno

from fastapi.testclient import TestClient
from httpx import AsyncClient

from qrkey.api import api
from qrkey.models import (
    MqttPinCodeModel,
)
from qrkey.settings import qrkey_settings


client = AsyncClient(app=api, base_url='http://testserver')

TEST_PIN_CODE = '123456789123'


@pytest.fixture(autouse=True)
def controller():
    api.controller = MagicMock()
    api.controller.pin_code = TEST_PIN_CODE
    api.controller.websockets = []


@pytest.mark.asyncio
async def test_openapi_exists():
    response = await client.get('/api')
    assert response.status_code == 200


@pytest.mark.asyncio
async def test_get_pin_code():
    response = await client.get('/pin_code')
    assert response.status_code == 200
    assert response.json() == MqttPinCodeModel(pin=TEST_PIN_CODE).model_dump()


@pytest.mark.asyncio
async def test_get_qr_code():
    buff = io.BytesIO()
    qrcode = segno.make_qr(f'{qrkey_settings.frontend_base_url}?pin={TEST_PIN_CODE}')
    qrcode.save(buff, kind='svg', scale=10, light=None)
    response = await client.get('/pin_code/qr_code')
    assert response.status_code == 200
    assert response.content == buff.getvalue()


@pytest.mark.asyncio
async def test_ws_client():
    with TestClient(api).websocket_connect('/ws') as websocket:
        await asyncio.sleep(0.1)
        assert len(api.controller.websockets) == 1
        websocket.close()
        await asyncio.sleep(0.1)
        assert len(api.controller.websockets) == 0



"""Module for the web server application."""

# SPDX-FileCopyrightText: 2024-present Alexandre Abadie <alexandre.abadie@inria.fr>
#
# SPDX-License-Identifier: BSD-3-Clause

import io
import os

import segno
from fastapi import FastAPI, WebSocket, WebSocketDisconnect
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import Response
from fastapi.staticfiles import StaticFiles

from qrkey.__about__ import __version__
from qrkey.models import (
    MqttPinCodeModel,
)
from qrkey.settings import qrkey_settings


api = FastAPI(
    debug=False,
    title='Qrkey controller API',
    description='This is the Qrkey controller API',
    version=__version__,
    docs_url='/api',
    redoc_url=None,
)
api.add_middleware(
    CORSMiddleware,
    allow_origins=['*'],
    allow_credentials=True,
    allow_methods=['*'],
    allow_headers=['*'],
)


@api.get(
    path='/pin_code',
    response_model=MqttPinCodeModel,
    summary='Return the current pin code',
    tags=['mqtt'],
)
async def pin_code():
    """Returns the pin code."""
    return MqttPinCodeModel(
        pin=api.controller.pin_code,
    )


@api.get(
    path='/pin_code/qr_code',
    response_class=Response,
    summary='Return the current QR code',
    tags=['mqtt'],
)
async def pin_code_qr_code():
    """Returns the QR code."""
    buff = io.BytesIO()
    qrcode = segno.make_qr(
        f'{qrkey_settings.frontend_base_url}?pin={api.controller.pin_code!s}'
    )
    qrcode.save(buff, kind='svg', scale=10, light=None)
    headers = {'Cache-Control': 'no-cache'}
    return Response(
        buff.getvalue().decode(),
        headers=headers,
        media_type='image/svg+xml',
    )


@api.websocket('/ws')
async def websocket_endpoint(websocket: WebSocket):
    """Websocket server endpoint."""
    await websocket.accept()
    api.controller.websockets.append(websocket)
    try:
        while True:
            _ = await websocket.receive_text()
    except WebSocketDisconnect:
        if websocket in api.controller.websockets:
            api.controller.websockets.remove(websocket)


# Mount static files after all routes are defined
PIN_CODE_DIR = os.path.join(os.path.dirname(__file__), 'ui', 'build')
api.mount('/pin', StaticFiles(directory=PIN_CODE_DIR, html=True), name='pin_code')
